X-Git-Url: https://git.xinqibao.xyz/dwm.git/blobdiff_plain/399993c6b5d594278bf822e2981ebfe8bfcb58c7..81aebf8caeb720406a238e8766bb615c501af15f:/main.c

diff --git a/main.c b/main.c
index c65cfc8..af1f1a6 100644
--- a/main.c
+++ b/main.c
@@ -1,4 +1,5 @@
 /* © 2004-2007 Anselm R. Garbe <garbeam at gmail dot com>
  * See LICENSE file for license details. */
 #include "dwm.h"
 #include <errno.h>
@@ -17,7 +18,7 @@
 
 char stext[256];
 int screen, sx, sy, sw, sh, wax, way, waw, wah;
-unsigned int bh, ntags, numlockmask;
+unsigned int bh, bpos, ntags, numlockmask;
 Atom wmatom[WMLast], netatom[NetLast];
 Bool *seltag;
 Bool selscreen = True;
@@ -80,7 +81,7 @@ initfont(const char *fontstr) {
 	dc.font.set = XCreateFontSet(dpy, fontstr, &missing, &n, &def);
 	if(missing) {
 		while(n--)
-			fprintf(stderr, "missing fontset: %s\n", missing[n]);
+			fprintf(stderr, "dwm: missing fontset: %s\n", missing[n]);
 		XFreeStringList(missing);
 	}
 	if(dc.font.set) {
@@ -162,9 +163,10 @@ setup(void) {
 	XFreeModifiermap(modmap);
 	/* select for events */
 	wa.event_mask = SubstructureRedirectMask | SubstructureNotifyMask
-		| EnterWindowMask | LeaveWindowMask;
+		| EnterWindowMask | LeaveWindowMask | StructureNotifyMask;
 	wa.cursor = cursor[CurNormal];
 	XChangeWindowAttributes(dpy, root, CWEventMask | CWCursor, &wa);
+	XSelectInput(dpy, root, wa.event_mask);
 	grabkeys();
 	compileregs();
 	for(ntags = 0; tags[ntags]; ntags++);
@@ -188,17 +190,14 @@ setup(void) {
 	wa.override_redirect = 1;
 	wa.background_pixmap = ParentRelative;
 	wa.event_mask = ButtonPressMask | ExposureMask;
-	barwin = XCreateWindow(dpy, root, sx, sy + (TOPBAR ? 0 : sh - bh), sw, bh, 0,
+	barwin = XCreateWindow(dpy, root, sx, sy, sw, bh, 0,
 			DefaultDepth(dpy, screen), CopyFromParent, DefaultVisual(dpy, screen),
 			CWOverrideRedirect | CWBackPixmap | CWEventMask, &wa);
 	XDefineCursor(dpy, barwin, cursor[CurNormal]);
+	bpos = BARPOS;
+	updatebarpos();
 	XMapRaised(dpy, barwin);
 	strcpy(stext, "dwm-"VERSION);
-	/* windowarea */
-	wax = sx;
-	way = sy + (TOPBAR ? bh : 0);
-	wah = sh - bh;
-	waw = sw;
 	/* pixmap for everything */
 	dc.drawable = XCreatePixmap(dpy, root, sw, bh, DefaultDepth(dpy, screen));
 	dc.gc = XCreateGC(dpy, root, 0, 0);
@@ -226,6 +225,32 @@ quit(const char *arg) {
 	readin = running = False;
 }
 
+void
+updatebarpos(void) {
+	XEvent ev;
+
+	wax = sx;
+	way = sy;
+	wah = sh;
+	waw = sw;
+	switch(bpos) {
+	default:
+		wah -= bh;
+		way += bh;
+		XMoveWindow(dpy, barwin, sx, sy);
+		break;
+	case BarBot:
+		wah -= bh;
+		XMoveWindow(dpy, barwin, sx, sy + wah);
+		break;
+	case BarOff:
+		XMoveWindow(dpy, barwin, sx, sy - bh);
+		break;
+	}
+	XSync(dpy, False);
+	while(XCheckMaskEvent(dpy, EnterWindowMask, &ev));
+}
+
 /* There's no way to check accesses to destroyed windows, thus those cases are
  * ignored (especially on UnmapNotify's).  Other types of errors call Xlibs
  * default error handler, which may call exit.
@@ -254,7 +279,7 @@ main(int argc, char *argv[]) {
 	XEvent ev;
 
 	if(argc == 2 && !strncmp("-v", argv[1], 3))
-		eprint("dwm-"VERSION", © 2004-2007 Anselm R. Garbe\n");
+		eprint("dwm-"VERSION", © 2004-2007 Anselm R. Garbe, Sander van Dijk\n");
 	else if(argc != 1)
 		eprint("usage: dwm [-v]\n");
 	setlocale(LC_CTYPE, "");
@@ -1,4 +1,5 @@
-/* © 2004-2007 Anselm R. Garbe <garbeam at gmail dot com>
+/* © 2006-2007 Anselm R. Garbe <garbeam at gmail dot com>
+ * © 2006-2007 Sander van Dijk <a dot h dot vandijk at gmail dot com>
  * See LICENSE file for license details. */
 #include "dwm.h"